@@ -1,4 +1,4 @@
-// $Id: exprwnd.cpp,v 1.10 1998/09/26 17:37:59 jsixt Exp $
+// $Id: exprwnd.cpp,v 1.17 1999/11/14 16:59:06 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -20,10 +20,10 @@ VarTree::VarTree(const QString& name, NameKind aKind) :
 	KTreeViewItem(name),
 	m_varKind(VKsimple),
 	m_nameKind(aKind),
-	m_value(100),			/* reserve some space */
 	m_valueChanged(false),
 	m_type(0),
-	m_exprIndex(0)
+	m_exprIndex(0),
+	m_exprIndexUseGuard(false)
 {
 }
 
@@ -129,7 +129,7 @@ bool VarTree::updateValue(const QString& newValue)
     return m_valueChanged || prevValueChanged;
 }
 
-void VarTree::inferTypesOfChildren()
+void VarTree::inferTypesOfChildren(ProgramTypeTable& typeTable)
 {
     /*
      * Type inference works like this: We use type information of those
@@ -140,14 +140,14 @@ void VarTree::inferTypesOfChildren()
     // first recurse children
     VarTree* child = static_cast<VarTree*>(getChild());
     while (child != 0) {
-	child->inferTypesOfChildren();
+	child->inferTypesOfChildren(typeTable);
 	child = static_cast<VarTree*>(child->getSibling());
     }
 
     // if this is a pointer, get the type from the value (less the pointer)
     if (m_varKind == VKpointer) {
-	char* p = m_value.data();
-	char* start = p;
+	const char* p = m_value.data();
+	const char* start = p;
 	// the type of the pointer shows up in the value (sometimes)
 	if (p == 0 || *p != '(')
 	    return;
@@ -162,18 +162,22 @@ void VarTree::inferTypesOfChildren()
 	    return;
 	}
 	const QString& typeName =
+#if QT_VERSION < 200
 	    QString(start+1, p-start-3+1) // minus 3 chars plus '\0'
+#else
+	    QString::fromLatin1(start+1, p-start-3) // minus 3 chars
+#endif
 		.stripWhiteSpace();
-	m_type = typeTable()[typeName];
+	m_type = typeTable.lookup(typeName);
 	if (m_type == 0) {
-	    m_type = TypeTable::unknownType();
+	    m_type = TypeInfo::unknownType();
 	}
     } else if (m_varKind == VKstruct) {
 	// check if this is a base class part
 	if (m_nameKind == NKtype) {
 	    const QString& typeName =
 		text.mid(1, text.length()-2); // strip < and >
-	    m_type = typeTable()[typeName];
+	    m_type = typeTable.lookup(typeName);
 
 	    /* if we don't have a type yet, get it from the base class */
 	    if (m_type == 0) {
@@ -188,7 +192,7 @@ void VarTree::inferTypesOfChildren()
 	     * If we still don't have a type, the type is really unknown.
 	     */
 	    if (m_type == 0) {
-		m_type = TypeTable::unknownType();
+		m_type = TypeInfo::unknownType();
 	    }
 	} // else
 	    /*
@@ -210,7 +214,7 @@ TypeInfo* VarTree::inferTypeFromBaseClass()
 	       child->m_nameKind == NKtype)
 	{
 	    if (child->m_type != 0 &&
-		child->m_type != TypeTable::unknownType())
+		child->m_type != TypeInfo::unknownType())
 	    {
 		// got a type!
 		return child->m_type;
@@ -232,8 +236,12 @@ ExprWnd::ExprWnd(QWidget* parent, const char* name) :
     connect(this, SIGNAL(expanded(int)), SLOT(slotExpandOrCollapse(int)));
     connect(this, SIGNAL(collapsed(int)), SLOT(slotExpandOrCollapse(int)));
 
+#if QT_VERSION < 200
     KIconLoader* loader = kapp->getIconLoader();
     m_pixPointer = loader->loadIcon("pointer.xpm");
+#else
+    m_pixPointer = BarIcon("pointer.xpm");
+#endif
     if (m_pixPointer.isNull())
 	TRACE("Can't load pointer.xpm");
 }
@@ -298,9 +306,6 @@ bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
 {
     bool isExpanded = display->isExpanded();
 
-    // display the new value
-    updateSingleExpr(display, newValues);
-
     /*
      * If we are updating a pointer without children by a dummy, we don't
      * collapse it, but simply insert the new children. This happens when a
@@ -335,6 +340,11 @@ bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
 	    collapseSubTree(display, false);
 	}
 
+	// since children changed, it is likely that the type has also changed
+	display->m_type = 0;		/* will re-evaluate the type */
+
+	// display the new value
+	updateSingleExpr(display, newValues);
 	replaceChildren(display, newValues);
 
 	// update the m_varKind
@@ -346,7 +356,10 @@ bool ExprWnd::updateExprRec(VarTree* display, VarTree* newValues)
 	// (note that the new value might not have a sub-tree at all)
 	return display->m_valueChanged || isExpanded;	/* no visible change if not expanded */
     }
-    
+
+    // display the new value
+    updateSingleExpr(display, newValues);
+
     /*
      * If this is an expanded pointer, record it for being updated.
      */
@@ -413,11 +426,10 @@ void ExprWnd::updateSingleExpr(VarTree* display, VarTree* newValue)
      */
     if (display->m_varKind == VarTree::VKstruct &&
 	display->m_type != 0 &&
-	display->m_type != TypeTable::unknownType())
+	display->m_type != TypeInfo::unknownType())
     {
 	ASSERT(newValue->m_varKind == VarTree::VKstruct);
 	display->m_partialValue = display->m_type->m_displayString[0];
-	display->m_exprIndex = 0;
 	m_updateStruct.append(display);
     }
     else
@@ -568,6 +580,14 @@ void ExprWnd::sweepList(QList<VarTree>& list, VarTree* subTree)
     }
 }
 
+QString ExprWnd::exprStringAt(int index)
+{
+    KTreeViewItem* item = itemAt(index);
+    if (item == 0) return QString();	/* paranoia */
+    VarTree* expr = static_cast<VarTree*>(item);
+    return expr->computeExpr();
+}
+
 void ExprWnd::clearPendingUpdates()
 {
     m_updatePtrs.clear();
