@@ -1,4 +1,4 @@
-// $Id: exprwnd.h,v 1.7 1998/08/10 21:01:11 jsixt Exp $
+// $Id: exprwnd.h,v 1.13 1999/12/18 13:39:12 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -8,7 +8,9 @@
 
 #include "ktreeview.h"
 #include <qlist.h>
+#include <qstrlist.h>
 
+class ProgramTypeTable;
 class TypeInfo;
 
 /* a variable's value is the tree of sub-variables */
@@ -21,13 +23,14 @@ public:
     };
     VarKind m_varKind;
     enum NameKind { NKplain, NKstatic, NKtype,
-	NKaddress,			/* a dereferenced pointer */
+	NKaddress			/* a dereferenced pointer */
     };
     NameKind m_nameKind;
     QString m_value;
     bool m_valueChanged;
     TypeInfo* m_type;			/* type of struct */
     int m_exprIndex;			/* used in struct value update */
+    bool m_exprIndexUseGuard;		/* ditto; if guard expr should be used */
     QString m_partialValue;		/* while struct value update is in progress */
 
     VarTree(const QString& name, NameKind kind);
@@ -42,7 +45,7 @@ public:
     /** update the value; return if repaint is necessary */
     bool updateValue(const QString& newValue);
     /** find out the type of this value using the child values */
-    void inferTypesOfChildren();
+    void inferTypesOfChildren(ProgramTypeTable& typeTable);
     /** get the type from base class part */
     TypeInfo* inferTypeFromBaseClass();
 };
@@ -69,6 +72,8 @@ public:
     VarTree* topLevelExprByName(const char* name);
     /** removes an expression; must be on the topmost level*/
     void removeExpr(VarTree* item);
+    /** retrieves the expression string at the specified line */
+    QString exprStringAt(int index);
     /** clears the list of pointers needing updates */
     void clearPendingUpdates();
     /** returns a pointer to update (or 0) and removes it from the list */
