@@ -1,5 +1,5 @@
 /*
- * $Id: ktreeview.h,v 1.7 1998/10/17 18:28:23 jsixt Exp $
+ * $Id: ktreeview.h,v 1.9 1999/12/11 18:17:46 jsixt Exp $
  * 
  * KTreeView class interface
  * 
@@ -28,7 +28,7 @@
 #include <qpixmap.h>			/* used in items */
 #include <qstack.h>			/* used to specify tree paths */
 #include <qstring.h>			/* used in items */
-#include <qtablevw.h>			/* base class for widget */
+#include <qtableview.h>			/* base class for widget */
 
 // use stack of strings to represent path information
 typedef QStack<QString> KPath;
@@ -37,6 +37,9 @@ class KTreeView;			/* forward declaration */
 
 /** Items for the KTreeView widget */
 class KTreeViewItem
+#if QT_VERSION >= 200
+: public Qt
+#endif
 {
     friend class KTreeView;
 public:
