@@ -1,10 +1,14 @@
-// $Id: procattach.cpp,v 1.1 1998/11/07 18:32:07 jsixt Exp $
+// $Id: procattach.cpp,v 1.3 1999/10/23 20:11:20 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
 
 #include "procattach.h"
 #include <kapp.h>
+#if QT_VERSION >= 200
+#include <klocale.h>			/* i18n */
+#endif
+#include "mydebug.h"
 
 ProcAttach::ProcAttach(QWidget* parent) :
 	QDialog(parent, "procattach", true),
