@@ -1,4 +1,4 @@
-// $Id: winstack.cpp,v 1.16 1998/10/11 20:08:04 jsixt Exp $
+// $Id: winstack.cpp,v 1.28 1999/12/11 18:17:46 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -8,15 +8,18 @@
 #include "commandids.h"
 #include "brkpt.h"
 #include <kfiledialog.h>
-#include <qpopmenu.h>
-#include <qtstream.h>
+#include <qtextstream.h>
 #include <qpainter.h>
 #include <qbrush.h>
 #include <qfile.h>
-#include <qfileinf.h>
+#include <qfileinfo.h>
 #include <qlistbox.h>
 #include <kapp.h>
 #include <kiconloader.h>
+#include <kstdaccel.h>
+#if QT_VERSION >= 200
+#include <klocale.h>			/* i18n */
+#endif
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -35,12 +38,22 @@ FileWindow::FileWindow(const char* fileName, QWidget* parent, const char* name)
     setFont(QFont("courier"));
 
     // load pixmaps
+#if QT_VERSION < 200
     KIconLoader* loader = kapp->getIconLoader();
     m_pcinner = loader->loadIcon("pcinner.xpm");
     m_pcup = loader->loadIcon("pcup.xpm");
     m_brkena = loader->loadIcon("brkena.xpm");
     m_brkdis = loader->loadIcon("brkdis.xpm");
     m_brktmp = loader->loadIcon("brktmp.xpm");
+    m_brkcond = loader->loadIcon("brkcond.xpm");
+#else
+    m_pcinner = BarIcon("pcinner");
+    m_pcup = BarIcon("pcup");
+    m_brkena = BarIcon("brkena");
+    m_brkdis = BarIcon("brkdis");
+    m_brktmp = BarIcon("brktmp");
+    m_brkcond = BarIcon("brkcond");
+#endif
 }
 
 FileWindow::~FileWindow()
@@ -77,7 +90,7 @@ void FileWindow::reloadFile()
 
     QTextStream t(&f);
     QString s;
-    uint lineNo = 0;
+    int lineNo = 0;
     while (lineNo < m_texts.size() && !t.eof()) {
 	s = t.readLine();
 	replaceLine(lineNo, s);
@@ -92,18 +105,20 @@ void FileWindow::reloadFile()
 	}
 	// allocate line items
 	m_lineItems.resize(m_texts.size());
-	for (uint i = m_texts.size()-1; i >= lineNo; i--) {
+	for (int i = m_texts.size()-1; i >= lineNo; i--) {
 	    m_lineItems[i] = 0;
 	}
     } else {
 	// the new file has fewer lines
 	// here lineNo is the number of lines of the new file
 	// remove the excessive lines
-	for (uint i = m_texts.size()-1; i >= lineNo; i--) {
-	    delete[] m_texts[i];
-	}
-	m_texts.resize(lineNo);
+	m_texts.setSize(lineNo);
 	m_lineItems.resize(lineNo);
+	// if the cursor is in the deleted lines, move it to the last line
+	if (m_curRow >= lineNo) {
+	    m_curRow = -1;		/* at this point don't have an active row */
+	    activateLine(lineNo-1);	/* now we have */
+	}
     }
     f.close();
 
@@ -162,8 +177,6 @@ void FileWindow::paintCell(QPainter* p, int row, int col)
 	uchar item = m_lineItems[row];
 	if (item == 0)			/* shortcut out */
 	    return;
-//	p->save();
-//	int w = 15;
 	int h = cellHeight(row);
 	if (item & liBP) {
 	    // enabled breakpoint
@@ -183,6 +196,12 @@ void FileWindow::paintCell(QPainter* p, int row, int col)
 	    if (y < 0) y = 0;
 	    p->drawPixmap(0,y,m_brktmp);
 	}
+	if (item & liBPconditional) {
+	    // conditional breakpoint marker
+	    int y = (h - m_brkcond.height())/2;
+	    if (y < 0) y = 0;
+	    p->drawPixmap(0,y,m_brkcond);
+	}
 	if (item & liPC) {
 	    // program counter in innermost frame
 	    int y = (h - m_pcinner.height())/2;
@@ -195,7 +214,6 @@ void FileWindow::paintCell(QPainter* p, int row, int col)
 	    if (y < 0) y = 0;
 	    p->drawPixmap(0,y,m_pcup);
 	}
-//	p->restore();
 	return;
     }
 }
@@ -231,22 +249,17 @@ void FileWindow::updateLineItems(const BreakpointTable& bpt)
 	    int i = bp.lineNo;
 	    if (i < 0 || uint(i) >= m_lineItems.size())
 		continue;
-	    if (bp.enabled) {
-		if (!(m_lineItems[i] & liBP)) {
-		    m_lineItems[i] &= ~liBPany;
-		    m_lineItems[i] |= liBP;
-		    if (bp.temporary)
-			m_lineItems[i] |= liBPtemporary;
-		    updateLineItem(i);
-		}
-	    } else {
-		if (!(m_lineItems[i] & liBPdisabled)) {
-		    m_lineItems[i] &= ~liBPany;
-		    m_lineItems[i] |= liBPdisabled;
-		    if (bp.temporary)
-			m_lineItems[i] |= liBPtemporary;
-		    updateLineItem(i);
-		}
+	    // compute new line item flags for breakpoint
+	    uchar flags = bp.enabled ? liBP : liBPdisabled;
+	    if (bp.temporary)
+		flags |= liBPtemporary;
+	    if (!bp.condition.isEmpty() || bp.ignoreCount != 0)
+		flags |= liBPconditional;
+	    // update if changed
+	    if ((m_lineItems[i] & liBPany) != flags) {
+		m_lineItems[i] &= ~liBPany;
+		m_lineItems[i] |= flags;
+		updateLineItem(i);
 	    }
 	}
     }
@@ -289,7 +302,11 @@ void FileWindow::find(const char* text, bool caseSensitive, FindDirection dir)
     if (line < 0)
 	line = 0;
     int curLine = line;			/* remember where we started */
+#if QT_VERSION < 200
     QString str;
+#else
+    QCString str;
+#endif
     bool found = false;
     do {
 	// advance and wrap around
@@ -312,11 +329,20 @@ void FileWindow::find(const char* text, bool caseSensitive, FindDirection dir)
     scrollTo(line);
 }
 
-void FileWindow::mouseReleaseEvent(QMouseEvent* ev)
+void FileWindow::mousePressEvent(QMouseEvent* ev)
 {
-    // check if event is in line item column
-    if (findCol(ev->x()) != 0)
+    // Check if right button was clicked.
+    if (ev->button() == RightButton)
+    {
+	emit clickedRight(ev->pos());
 	return;
+    }
+
+    // check if event is in line item column
+    if (findCol(ev->x()) != 0) {
+	KTextView::mousePressEvent(ev);
+	return;
+    }
 
     // get row
     int row = findRow(ev->y());
@@ -338,24 +364,50 @@ void FileWindow::mouseReleaseEvent(QMouseEvent* ev)
 
 
 
-WinStack::WinStack(QWidget* parent, const char* name, const BreakpointTable& bpt) :
+WinStack::WinStack(QWidget* parent, const char* name) :
 	QWidget(parent, name),
 	m_activeWindow(0),
 	m_windowMenu(0),
 	m_itemMore(0),
-	m_pcLine(-1),
-	m_bpTable(bpt)
+	m_pcLine(-1)
 {
+    // Call menu implementation helper
+    initMenu();
+
     connect(&m_findDlg.m_buttonForward,
 	    SIGNAL(clicked()), SLOT(slotFindForward()));
     connect(&m_findDlg.m_buttonBackward,
 	    SIGNAL(clicked()), SLOT(slotFindBackward()));
+
+    // Check for right click event.
+    connect(this, SIGNAL(clickedRight(const QPoint &)),
+	    SLOT(slotWidgetRightClick(const QPoint &)));
 }
 
 WinStack::~WinStack()
 {
 }
 
+// All menu initializations.
+void WinStack::initMenu()
+{
+    // Init float popup menu.
+    m_menuFloat.insertItem(i18n("&Open Source..."), ID_FILE_OPEN);
+    m_menuFloat.insertSeparator();
+    m_menuFloat.insertItem(i18n("Step &into"), ID_PROGRAM_STEP);
+    m_menuFloat.insertItem(i18n("Step &over"), ID_PROGRAM_NEXT);
+    m_menuFloat.insertItem(i18n("Step o&ut"), ID_PROGRAM_FINISH);
+    m_menuFloat.insertItem(i18n("Run to &cursor"), ID_PROGRAM_UNTIL);
+    m_menuFloat.insertSeparator();
+    m_menuFloat.insertItem(i18n("Set/Clear &breakpoint"), ID_BRKPT_SET);
+
+    // Init float file popup.
+    m_menuFileFloat.insertItem(i18n("&Open Source..."), ID_FILE_OPEN);
+    m_menuFileFloat.insertSeparator();
+    m_menuFileFloat.insertItem(i18n("&Executable..."), ID_FILE_EXECUTABLE);
+    m_menuFileFloat.insertItem(i18n("&Core dump..."), ID_FILE_COREFILE);
+}
+
 void WinStack::setWindowMenu(QPopupMenu* menu)
 {
     m_windowMenu = menu;
@@ -402,7 +454,7 @@ void WinStack::menuCallback(int item)
 
 void WinStack::openFile()
 {
-    QString fileName = KFileDialog::getOpenFileName(m_lastOpenDir);
+    QString fileName = KFileDialog::getOpenFileName(m_lastOpenDir, QString(), this);
     TRACE("openFile: " + fileName);
     if (fileName.isEmpty()) {
 	return;
@@ -414,6 +466,18 @@ void WinStack::openFile()
     activateFI(fi, 0);
 }
 
+void WinStack::mousePressEvent(QMouseEvent* mouseEvent)
+{
+    // Check if right button was clicked.
+    if (mouseEvent->button() == RightButton)
+    {
+	emit clickedRight(mouseEvent->pos());
+    } else {
+	QWidget::mousePressEvent(mouseEvent);
+    }
+}
+
+
 void WinStack::reloadAllFiles()
 {
     FileWindow* fw;
@@ -422,10 +486,10 @@ void WinStack::reloadAllFiles()
     }
 }
 
-bool WinStack::activate(QString fileName, int lineNo)
+void WinStack::activate(const QString& fileName, int lineNo)
 {
     QFileInfo fi(fileName);
-    return activateFI(fi, lineNo);
+    activateFI(fi, lineNo);
 }
 
 bool WinStack::activateFI(QFileInfo& fi, int lineNo)
@@ -461,11 +525,15 @@ bool WinStack::activatePath(QString pathName, int lineNo)
 	// not found, load it
 	fw = new FileWindow(pathName, this, "fileWindow");
 	m_fileList.insert(0, fw);
-	connect(fw, SIGNAL(lineChanged()),SLOT(slotLineChanged()));
+	connect(fw, SIGNAL(lineChanged()),SIGNAL(lineChanged()));
 	connect(fw, SIGNAL(clickedLeft(const QString&, int)),
-		SLOT(slotToggleBreak(const QString&,int)));
+		SIGNAL(toggleBreak(const QString&,int)));
 	connect(fw, SIGNAL(clickedMid(const QString&, int)),
-		SLOT(slotEnaDisBreak(const QString&,int)));
+		SIGNAL(enadisBreak(const QString&,int)));
+
+	// Comunication when right button is clicked.
+	connect(fw, SIGNAL(clickedRight(const QPoint &)),
+		SLOT(slotFileWindowRightClick(const QPoint &)));
 
 	changeWindowMenu();
 	
@@ -473,7 +541,7 @@ bool WinStack::activatePath(QString pathName, int lineNo)
 	fw->loadFile();
 	
 	// set PC if there is one
-	updateLineItems();
+	emit newFileLoaded();
 	if (m_pcLine >= 0) {
 	    setPC(true, m_pcFile, m_pcLine, m_pcFrame);
 	}
@@ -545,7 +613,7 @@ void WinStack::changeWindowMenu()
     while ((m_windowMenu->idAt(m_itemMore) & ~ID_WINDOW_INDEX_MASK) == ID_WINDOW_MORE) {
 	m_windowMenu->removeItemAt(m_itemMore);
     }
-    
+
     // insert current windows
     QString text;
     int index = 1;
@@ -562,11 +630,11 @@ void WinStack::changeWindowMenu()
     }
 }
 
-void WinStack::updateLineItems()
+void WinStack::updateLineItems(const BreakpointTable& bpt)
 {
     FileWindow* fw = 0;
     for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
-	fw->updateLineItems(m_bpTable);
+	fw->updateLineItems(bpt);
     }
 }
 
@@ -605,8 +673,8 @@ static bool fileNamesMatch(const QString& f1, const QString& f2)
 
 void WinStack::setPC(bool set, const QString& fileName, int lineNo, int frameNo)
 {
-    TRACE(QString(fileName.length()+60).sprintf("%s PC: %s:%d#%d", set ? "set" : "clear",
-						fileName.data(), lineNo, frameNo));
+    TRACE((set ? "set PC: " : "clear PC: ") + fileName +
+	  QString().sprintf(":%d#%d", lineNo, frameNo));
     // find file
     FileWindow* fw = 0;
     for (fw = m_fileList.first(); fw != 0; fw = m_fileList.next()) {
@@ -625,11 +693,6 @@ void WinStack::resizeEvent(QResizeEvent*)
     }
 }
 
-void WinStack::slotLineChanged()
-{
-    emit lineChanged();
-}
-
 void WinStack::slotFindForward()
 {
     if (m_activeWindow != 0)
@@ -644,14 +707,28 @@ void WinStack::slotFindBackward()
 			     FileWindow::findBackward);
 }
 
-void WinStack::slotToggleBreak(const QString& fileName, int row)
+void WinStack::slotFileWindowRightClick(const QPoint & pos)
 {
-    emit toggleBreak(fileName, row);
+    if (m_menuFloat.isVisible())
+    {
+	m_menuFloat.hide();
+    }
+    else
+    {
+	m_menuFloat.popup(mapToGlobal(pos));
+    }
 }
 
-void WinStack::slotEnaDisBreak(const QString& fileName, int row)
+void WinStack::slotWidgetRightClick(const QPoint & pos)
 {
-    emit enadisBreak(fileName, row);
+    if (m_menuFileFloat.isVisible())
+    {
+	m_menuFileFloat.hide();
+    }
+    else
+    {
+	m_menuFileFloat.popup(mapToGlobal(pos));
+    }
 }
 
 
