@@ -1,4 +1,4 @@
-// $Id: winstack.h,v 1.12 1998/10/11 20:08:05 jsixt Exp $
+// $Id: winstack.h,v 1.21 1999/12/11 18:17:46 jsixt Exp $
 
 // Copyright by Johannes Sixt
 // This file is under GPL, the GNU General Public Licence
@@ -6,14 +6,15 @@
 #ifndef WINSTACK_H
 #define WINSTACK_H
 
-#include <qmlined.h>
+#include <qmultilinedit.h>
 #include <qlist.h>
 #include <qpixmap.h>
 #include <qdialog.h>
-#include <qlined.h>
+#include <qlineedit.h>
 #include <qlayout.h>
-#include <qchkbox.h>
-#include <qpushbt.h>
+#include <qcheckbox.h>
+#include <qpushbutton.h>
+#include <qpopupmenu.h>
 #include "textvw.h"
 
 // forward declarations
@@ -42,18 +43,20 @@ protected:
     virtual int textCol() const;
     virtual int cellWidth(int col);
     virtual void paintCell(QPainter* p, int row, int col);
-    virtual void mouseReleaseEvent(QMouseEvent* ev);
+    virtual void mousePressEvent(QMouseEvent* ev);
     void updateLineItem(int i);
 
 signals:
     void clickedLeft(const QString&, int);
     void clickedMid(const QString&, int);
+    void clickedRight(const QPoint &);
 
 protected:
     QString m_fileName;
     enum LineItem { liPC = 1, liPCup = 2,
 	liBP = 4, liBPdisabled = 8, liBPtemporary = 16,
-	liBPany = liBP|liBPdisabled|liBPtemporary
+	liBPconditional = 32,
+	liBPany = liBP|liBPdisabled|liBPtemporary|liBPconditional
     };
     QArray<uchar> m_lineItems;
     QPixmap m_pcinner;			/* PC at innermost frame */
@@ -61,6 +64,7 @@ protected:
     QPixmap m_brkena;			/* enabled breakpoint */
     QPixmap m_brkdis;			/* disabled breakpoint */
     QPixmap m_brktmp;			/* temporary breakpoint marker */
+    QPixmap m_brkcond;			/* conditional breakpoint marker */
 };
 
 class FindDialog : public QDialog
@@ -93,7 +97,7 @@ class WinStack : public QWidget
 {
     Q_OBJECT
 public:
-    WinStack(QWidget* parent, const char* name, const BreakpointTable& bpt);
+    WinStack(QWidget* parent, const char* name);
     virtual ~WinStack();
     
     /**
@@ -104,11 +108,7 @@ public:
      */
     void setWindowMenu(QPopupMenu* menu);
     void selectWindow(int index);	/* 1-based index, 0 means dialog More... */
-    bool activate(QString filename, int lineNo);
     bool activeLine(QString& filename, int& lineNo);
-    void updateLineItems();
-    void updatePC(const QString& filename, int lineNo, int frameNo);
-    void reloadAllFiles();
 
     virtual void resizeEvent(QResizeEvent*);
 
@@ -117,21 +117,30 @@ signals:
     void lineChanged();
     void toggleBreak(const QString&, int);
     void enadisBreak(const QString&, int);
+    void clickedRight(const QPoint&);
+    void newFileLoaded();
 
 public slots:
     virtual void menuCallback(int item);
-    virtual void slotLineChanged();
     virtual void slotFindForward();
     virtual void slotFindBackward();
-    virtual void slotToggleBreak(const QString&,int);
-    virtual void slotEnaDisBreak(const QString&,int);
+    virtual void slotFileWindowRightClick(const QPoint &);
+    virtual void activate(const QString& filename, int lineNo);
+    void updatePC(const QString& filename, int lineNo, int frameNo);
+    void reloadAllFiles();
+    void updateLineItems(const BreakpointTable& bpt);
+
+    // Right click on file panner when no file is loaded.
+    virtual void slotWidgetRightClick(const QPoint &);
 
 protected:
+    void initMenu();
     bool activateFI(QFileInfo& fi, int lineNo);
     bool activatePath(QString pathname, int lineNo);
     virtual bool activateWindow(FileWindow* fw, int lineNo = -1);	/* -1 doesnt change line */
     virtual void changeWindowMenu();
     virtual void openFile();
+    virtual void mousePressEvent(QMouseEvent*);
     void setPC(bool set, const QString& fileName, int lineNo, int frameNo);
     QList<FileWindow> m_fileList;
     FileWindow* m_activeWindow;
@@ -144,11 +153,14 @@ protected:
     QString m_pcFile;
     int m_pcLine;			/* -1 if no PC */
     int m_pcFrame;
-    const BreakpointTable& m_bpTable;
 
 public:
     // find dialog
     FindDialog m_findDlg;
+
+    // Popup menu
+    QPopupMenu m_menuFloat;
+    QPopupMenu m_menuFileFloat;
 };
 
 #endif // WINSTACK_H
